#include "testApp.h"
//--------------------------------------------------------------
void testApp::setup(){
	vidGrabber.setVerbose(true);
	vidGrabber.initGrabber(320,240);
	colorImg.allocate(320,240);
	grayImage.allocate(320,240);
	grayBg.allocate(320,240);
	grayDiff.allocate(320,240);
	bLearnBackground = true;
	threshold = 80;
	bHoles = true;
	offset = 0;
	spacing = 10;
	numberBlobs = 2;
	ofBackground(50, 50, 50);
}
//--------------------------------------------------------------
void testApp::update(){
	vidGrabber.grabFrame();
	if(vidGrabber.isFrameNew()){
		//1. color image
		colorImg.setFromPixels(vidGrabber.getPixels(),320,240);
		//2. gray image
		grayImage = colorImg;
		//3. background
		if(bLearnBackground==true){
			grayBg = grayImage;
			bLearnBackground = false;
		}
		//4. difference
		grayDiff.absDiff(grayImage, grayBg);
		grayDiff.threshold(threshold);
		grayDiff.blur(5);
		//5. contours > blobs
		contourFinder.findContours(grayDiff,20,320*240/3,numberBlobs,bHoles);
	}
}
//--------------------------------------------------------------
void testApp::draw(){
	ofSetColor(255, 255, 255);
	if(offset==0){
		colorImg.draw(spacing,spacing-offset);
		grayImage.draw(spacing*2+320,spacing-offset);
		grayBg.draw(spacing*3+320*2,spacing-offset);
		grayDiff.draw(spacing,spacing*2+240-offset);
		contourFinder.draw(spacing,spacing*2+240-offset);
	}else {
		grayDiff.draw(spacing,spacing*2+240-offset,160,120);
		contourFinder.draw(spacing,spacing*2+240-offset,160,120);
	}
	string strThreshold = ofToString(threshold);
	ofDrawBitmapString("'space' > set background; 'up'/'down' keys > threshold: "
					   +strThreshold+"; 'backspace' > holes: "
					   +ofToString(bHoles)
					   +"; 'return' > hide top images"
					   +"; blobs: "+ ofToString(numberBlobs)
					   , spacing, ofGetHeight()-spacing);
	for (int i = 0; i < contourFinder.blobs.size(); i++) {
		float blobX[contourFinder.blobs.size()];
		float blobY[contourFinder.blobs.size()];
		blobX[i] = (1-contourFinder.blobs[i].centroid.x/320)*ofGetWidth();
		blobY[i] = contourFinder.blobs[i].centroid.y/240*ofGetHeight();
		//draw blob shapes
		ofSetColor(255, 255, 255);
		ofBeginShape();
		for (int j = 0; j < contourFinder.blobs[i].nPts; j++){
			ofVertex((1-contourFinder.blobs[i].pts[j].x/320)*ofGetWidth(), 
					 + contourFinder.blobs[i].pts[j].y/240*ofGetHeight());
		}
		ofEndShape(true);	
		//draw circles
		ofSetColor(255, 0, 0);
		ofCircle(blobX[i],blobY[i],10);
	}
}
//--------------------------------------------------------------
void testApp::keyPressed(int key){
	if (key == 'p') {
		string dateStamp = "_"+ofToString(ofGetYear())+"-"+ofToString(ofGetMonth())+"-"+ofToString(ofGetDay())+"_"+ofToString(ofGetHours())+"-"+ofToString(ofGetMinutes())+"_"+ofToString(ofGetSeconds());
		ofSaveScreen("screenshot"+dateStamp+".png");
	}	
	if (key == ' '){
		bLearnBackground = true;
	}
	if (key == OF_KEY_UP) {
		threshold++;
	}
	if (key == OF_KEY_DOWN) {
		if(threshold>1){
			threshold--;
		}
	}
	if(key == OF_KEY_BACKSPACE){
		if (bHoles) {
			bHoles = false;
		}else {
			bHoles = true;
		}
	}
	if(key == OF_KEY_RETURN){
		if (offset) {
			offset = 0;
		}else {
			offset = 240+spacing;
		}
	}
	
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

